@@ -1,23 +1,12 @@
 """
 AMICI
 -----
-The AMICI Python module provides functionality for importing SBML models and
-turning them into C++ Python extensions.
 
-Getting started:
-```
-# creating a extension module for an SBML model:
-import amici
-amiSbml = amici.SbmlImporter('mymodel.sbml')
-amiSbml.sbml2amici('modelName', 'outputDirectory')
-
-# using the created module (set python path)
-import modelName
-help(modelName)
-```
+The AMICI Python module provides functionality for importing SBML or PySB
+models and turning them into C++ Python extensions.
 
 :var amici_path:
-    absolute root path of the amici repository
+    absolute root path of the amici repository or Python package
 :var amiciSwigPath:
     absolute path of the amici swig directory
 :var amiciSrcPath:
@@ -147,31 +136,54 @@ if not _imported_from_setup():
 hdf5_enabled = 'readSolverSettingsFromHDF5' in dir()
 
 
+def _get_ptr(obj: Union[AmiciModel, AmiciExpData, AmiciSolver]
+             ) -> Union['amici.Model', 'amici.ExpData', 'amici.Solver']:
+    """
+    Convenience wrapper that returns the smart pointer pointee, if applicable
+
+    :param obj:
+        Potential smart pointer
+
+    :returns:
+        Non-smart pointer
+    """
+    if isinstance(obj, (amici.ModelPtr, amici.ExpDataPtr, amici.SolverPtr)):
+        return obj.get()
+    return obj
+
+
 def runAmiciSimulation(
         model: AmiciModel,
         solver: AmiciSolver,
         edata: Optional[AmiciExpData] = None
 ) -> 'numpy.ReturnDataView':
     """
-    Convenience wrapper around amici.runAmiciSimulation (generated by swig)
+    Convenience wrapper around :py:func:`amici.amici.runAmiciSimulation`
+    (generated by swig)
 
-    :param model: Model instance
-`   :param solver: Solver instance, must be generated from Model.getSolver()
-    :param edata: ExpData instance (optional)
+    :param model:
+        Model instance
 
-    :returns: ReturnData object with simulation results
+`   :param solver:
+        Solver instance, must be generated from
+        :py:meth:`amici.amici.Model.getSolver`
+
+    :param edata:
+        ExpData instance (optional)
+
+    :returns:
+        ReturnData object with simulation results
     """
-    if edata and isinstance(edata, amici.ExpDataPtr):
-        edata = edata.get()
 
     with capture_cstdout():
-        rdata = amici.runAmiciSimulation(solver.get(), edata, model.get())
+        rdata = amici.runAmiciSimulation(_get_ptr(solver), _get_ptr(edata),
+                                         _get_ptr(model))
     return numpy.ReturnDataView(rdata)
 
 
 def ExpData(*args) -> 'amici.ExpData':
     """
-    Convenience wrapper for ExpData constructors
+    Convenience wrapper for :py:class:`amici.amici.ExpData` constructors
 
     :param args: arguments
 
@@ -179,13 +191,13 @@ def ExpData(*args) -> 'amici.ExpData':
     """
     if isinstance(args[0], ReturnDataView):
         return amici.ExpData(args[0]['ptr'].get(), *args[1:])
-    elif isinstance(args[0], amici.ExpDataPtr):
+    elif isinstance(args[0], (amici.ExpData, amici.ExpDataPtr)):
         # the *args[:1] should be empty, but by the time you read this,
         # the constructor signature may have changed and you are glad this
         # wrapper did not break.
-        return amici.ExpData(args[0].get(), *args[1:])
-    elif isinstance(args[0], amici.ModelPtr):
-        return amici.ExpData(args[0].get())
+        return amici.ExpData(_get_ptr(args[0]), *args[1:])
+    elif isinstance(args[0], (amici.Model, amici.ModelPtr)):
+        return amici.ExpData(_get_ptr(args[0]))
     else:
         return amici.ExpData(*args)
 
@@ -211,9 +223,9 @@ def runAmiciSimulations(
     """
     with capture_cstdout():
         edata_ptr_vector = amici.ExpDataPtrVector(edata_list)
-        rdata_ptr_list = amici.runAmiciSimulations(solver.get(),
+        rdata_ptr_list = amici.runAmiciSimulations(_get_ptr(solver),
                                                    edata_ptr_vector,
-                                                   model.get(),
+                                                   _get_ptr(model),
                                                    failfast,
                                                    num_threads)
     return [numpy.ReturnDataView(r) for r in rdata_ptr_list]
@@ -225,16 +237,13 @@ def readSolverSettingsFromHDF5(
         location: Optional[str] = 'solverSettings'
 ) -> None:
     """
-    Convenience wrapper for :fun:`amici.readSolverSettingsFromHDF5`
+    Convenience wrapper for :py:func:`amici.readSolverSettingsFromHDF5`
 
     :param file: hdf5 filename
     :param solver: Solver instance to which settings will be transferred
     :param location: location of solver settings in hdf5 file
     """
-    if isinstance(solver, amici.SolverPtr):
-        amici.readSolverSettingsFromHDF5(file, solver.get(), location)
-    else:
-        amici.readSolverSettingsFromHDF5(file, solver, location)
+    amici.readSolverSettingsFromHDF5(file, _get_ptr(solver), location)
 
 
 def writeSolverSettingsToHDF5(
@@ -243,17 +252,14 @@ def writeSolverSettingsToHDF5(
         location: Optional[str] = 'solverSettings'
 ) -> None:
     """
-    Convenience wrapper for :fun:`amici.writeSolverSettingsToHDF5`
+    Convenience wrapper for :py:func:`amici.amici.writeSolverSettingsToHDF5`
 
     :param file: hdf5 filename, can also be object created by
-        :fun:`amici.createOrOpenForWriting`
+        :py:func:`amici.amici.createOrOpenForWriting`
     :param solver: Solver instance from which settings will stored
     :param location: location of solver settings in hdf5 file
     """
-    if isinstance(solver, amici.SolverPtr):
-        amici.writeSolverSettingsToHDF5(solver.get(), file, location)
-    else:
-        amici.writeSolverSettingsToHDF5(solver, file, location)
+    amici.writeSolverSettingsToHDF5(_get_ptr(solver), file, location)
 
 
 class add_path:
@@ -278,9 +284,12 @@ def import_model_module(module_name: str,
     """
     Import Python module of an AMICI model
 
-    :param module_name: Name of the python package of the model
-    :param module_path: Absolute or relative path of the package directory
-    :return: The model module
+    :param module_name:
+        Name of the python package of the model
+    :param module_path:
+        Absolute or relative path of the package directory
+    :return:
+        The model module
     """
 
     # ensure we will find the newly created module
