@@ -22,7 +22,7 @@ class IDASolver;
  *
  * The model does not contain any data, but represents the state
  * of the model at a specific time t. The states must not always be
- * in sync, but may be updated asynchroneously.
+ * in sync, but may be updated asynchronously.
  */
 class Model_DAE : public Model {
   public:
@@ -49,6 +49,8 @@ class Model_DAE : public Model {
      * repeating elements
      * @param ndwdp number of nonzero elements in the p derivative of the
      * repeating elements
+     * @param ndwdw number of nonzero elements in the w derivative of the
+     * repeating elements
      * @param ndxdotdw number of nonzero elements dxdotdw
      * @param ndJydy number of nonzero elements dJydy
      * @param nnz number of nonzero elements in Jacobian
@@ -62,22 +64,23 @@ class Model_DAE : public Model {
      * @param z2event mapping of event outputs to events
      * @param pythonGenerated flag indicating matlab or python wrapping
      * @param ndxdotdp_explicit number of nonzero elements dxdotdp_explicit
-     * @param ndxdotdp_implicit number of nonzero elements dxdotdp_implicit
      */
     Model_DAE(const int nx_rdata, const int nxtrue_rdata, const int nx_solver,
               const int nxtrue_solver, const int nx_solver_reinit, const int ny, const int nytrue,
               const int nz, const int nztrue, const int ne, const int nJ,
-              const int nw, const int ndwdx, const int ndwdp, const int ndxdotdw,
-              std::vector<int> ndJydy, const int nnz, const int ubw,
-              const int lbw, const SecondOrderMode o2mode,
+              const int nw, const int ndwdx, const int ndwdp, const int ndwdw,
+              const int ndxdotdw, std::vector<int> ndJydy, const int nnz,
+              const int ubw, const int lbw, const SecondOrderMode o2mode,
               std::vector<realtype> const &p, std::vector<realtype> const &k,
-              std::vector<int> const &plist, std::vector<realtype> const &idlist,
+              std::vector<int> const &plist,
+              std::vector<realtype> const &idlist,
               std::vector<int> const &z2event, const bool pythonGenerated=false,
-              const int ndxdotdp_explicit=0, const int ndxdotdp_implicit=0)
-        : Model(nx_rdata, nxtrue_rdata, nx_solver, nxtrue_solver, nx_solver_reinit, ny, nytrue,
-                nz, nztrue, ne, nJ, nw, ndwdx, ndwdp, ndxdotdw, std::move(ndJydy),
-                nnz, ubw, lbw, o2mode, p, k, plist, idlist, z2event,
-                pythonGenerated, ndxdotdp_explicit, ndxdotdp_implicit) {
+              const int ndxdotdp_explicit=0)
+        : Model(nx_rdata, nxtrue_rdata, nx_solver, nxtrue_solver,
+                nx_solver_reinit, ny, nytrue, nz, nztrue, ne, nJ, nw, ndwdx,
+                ndwdp, ndwdw, ndxdotdw, std::move(ndJydy), nnz, ubw, lbw,
+                o2mode, p, k, plist, idlist, z2event, pythonGenerated,
+                ndxdotdp_explicit) {
             M_ = SUNMatrixWrapper(nx_solver, nx_solver);
         }
 
@@ -320,45 +323,6 @@ class Model_DAE : public Model {
     std::unique_ptr<Solver> getSolver() override;
 
   protected:
-    /**
-     * @brief Model specific implementation for fJ
-     * @param J Matrix to which the Jacobian will be written
-     * @param t timepoint
-     * @param x Vector with the states
-     * @param p parameter vector
-     * @param k constants vector
-     * @param h heavyside vector
-     * @param cj scaling factor, inverse of the step size
-     * @param dx Vector with the derivative states
-     * @param w vector with helper variables
-     * @param dwdx derivative of w wrt x
-     **/
-    virtual void fJ(realtype *J, realtype t, const realtype *x, const double *p,
-                    const double *k, const realtype *h, realtype cj,
-                    const realtype *dx, const realtype *w,
-                    const realtype *dwdx) = 0;
-
-    /**
-     * @brief Model specific implementation for fJB
-     * @param JB Matrix to which the Jacobian will be written
-     * @param t timepoint
-     * @param x Vector with the states
-     * @param p parameter vector
-     * @param k constants vector
-     * @param h heavyside vector
-     * @param cj scaling factor, inverse of the step size
-     * @param xB Vector with the adjoint states
-     * @param dx Vector with the derivative states
-     * @param dxB Vector with the adjoint derivative states
-     * @param w vector with helper variables
-     * @param dwdx derivative of w wrt x
-     **/
-    virtual void fJB(realtype *JB, realtype t, const realtype *x,
-                     const double *p, const double *k, const realtype *h,
-                     realtype cj, const realtype *xB, const realtype *dx,
-                     const realtype *dxB, const realtype *w,
-                     const realtype *dwdx);
-
     /**
      * @brief Model specific implementation for fJSparse
      * @param JSparse Matrix to which the Jacobian will be written
@@ -366,7 +330,7 @@ class Model_DAE : public Model {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h Heaviside vector
      * @param cj scaling factor, inverse of the step size
      * @param dx Vector with the derivative states
      * @param w vector with helper variables
@@ -377,68 +341,6 @@ class Model_DAE : public Model {
                           const realtype *h, realtype cj, const realtype *dx,
                           const realtype *w, const realtype *dwdx) = 0;
 
-    /**
-     * @brief Model specific implementation for fJSparseB
-     * @param JSparseB Matrix to which the Jacobian will be written
-     * @param t timepoint
-     * @param x Vector with the states
-     * @param p parameter vector
-     * @param k constants vector
-     * @param h heavyside vector
-     * @param cj scaling factor, inverse of the step size
-     * @param xB Vector with the adjoint states
-     * @param dx Vector with the derivative states
-     * @param dxB Vector with the adjoint derivative states
-     * @param w vector with helper variables
-     * @param dwdx derivative of w wrt x
-     **/
-    virtual void fJSparseB(SUNMatrixContent_Sparse JSparseB, const realtype t,
-                           const realtype *x, const double *p, const double *k,
-                           const realtype *h, const realtype cj,
-                           const realtype *xB, const realtype *dx,
-                           const realtype *dxB, const realtype *w,
-                           const realtype *dwdx);
-
-    /**
-     * @brief Model specific implementation for fJDiag
-     * @param JDiag array to which the Jacobian diagonal will be written
-     * @param t timepoint
-     * @param x Vector with the states
-     * @param p parameter vector
-     * @param k constants vector
-     * @param h heavyside vector
-     * @param cj scaling factor, inverse of the step size
-     * @param dx Vector with the derivative states
-     * @param w vector with helper variables
-     * @param dwdx derivative of w wrt x
-     **/
-    virtual void fJDiag(realtype *JDiag, realtype t, const realtype *x,
-                        const realtype *p, const realtype *k, const realtype *h,
-                        realtype cj, const realtype *dx, const realtype *w,
-                        const realtype *dwdx);
-
-    /**
-     * @brief Model specific implementation for fJvB
-     * @param JvB Matrix vector product of JB with a vector v
-     * @param t timepoint
-     * @param x Vector with the states
-     * @param p parameter vector
-     * @param k constants vector
-     * @param h heavyside vector
-     * @param cj scaling factor, inverse of the step size
-     * @param xB Vector with the adjoint states
-     * @param dx Vector with the derivative states
-     * @param dxB Vector with the adjoint derivative states
-     * @param vB Vector with which the Jacobian is multiplied
-     * @param w vector with helper variables
-     * @param dwdx derivative of w wrt x
-     **/
-    virtual void fJvB(realtype *JvB, realtype t, const realtype *x,
-                      const double *p, const double *k, const realtype *h,
-                      realtype cj, const realtype *xB, const realtype *dx,
-                      const realtype *dxB, const realtype *vB,
-                      const realtype *w, const realtype *dwdx);
-
     /**
      * @brief Model specific implementation for froot
      * @param root values of the trigger function
@@ -446,7 +348,7 @@ class Model_DAE : public Model {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h Heaviside vector
      * @param dx Vector with the derivative states
      **/
     virtual void froot(realtype *root, realtype t, const realtype *x,
@@ -460,7 +362,7 @@ class Model_DAE : public Model {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h Heaviside vector
      * @param w vector with helper variables
      * @param dx Vector with the derivative states
      **/
@@ -475,7 +377,7 @@ class Model_DAE : public Model {
      * @param x Vector with the states
      * @param p parameter vector
      * @param k constants vector
-     * @param h heavyside vector
+     * @param h Heaviside vector
      * @param ip parameter index
      * @param dx Vector with the derivative states
      * @param w vector with helper variables
