@@ -13,10 +13,10 @@ void Model_ODE::fJ(const realtype t, const realtype /*cj*/, const AmiVector &x,
 void Model_ODE::fJ(realtype t, N_Vector x, N_Vector /*xdot*/, SUNMatrix J) {
     auto x_pos = computeX_pos(x);
     fdwdx(t, N_VGetArrayPointer(x_pos));
-    SUNMatZero(J);
-    fJ(SM_DATA_D(J), t, N_VGetArrayPointer(x_pos),
-       state_.unscaledParameters.data(), state_.fixedParameters.data(),
-       state_.h.data(), w_.data(), dwdx_.data());
+    fJSparse(t, x, J_.get());
+    J_.refresh();
+    auto JDense = SUNMatrixWrapper(J);
+    J_.to_dense(JDense);
 }
 
 void Model_ODE::fJSparse(const realtype t, const realtype /*cj*/,
@@ -28,13 +28,25 @@ void Model_ODE::fJSparse(const realtype t, const realtype /*cj*/,
 void Model_ODE::fJSparse(realtype t, N_Vector x, SUNMatrix J) {
     auto x_pos = computeX_pos(x);
     fdwdx(t, N_VGetArrayPointer(x_pos));
-    SUNMatZero(J);
     if (pythonGenerated) {
-        fJSparse(SM_DATA_S(J), t, N_VGetArrayPointer(x_pos),
-                 state_.unscaledParameters.data(), state_.fixedParameters.data(),
-                 state_.h.data(), w_.data(), dwdx_.data());
-        fJSparse_colptrs(SM_INDEXPTRS_S(J));
-        fJSparse_rowvals(SM_INDEXVALS_S(J));
+        auto JSparse = SUNMatrixWrapper(J);
+        // python generated
+        dxdotdx_explicit.zero();
+        dxdotdx_implicit.zero();
+        if (dxdotdx_explicit.capacity()) {
+            fdxdotdx_explicit_colptrs(dxdotdx_explicit);
+            fdxdotdx_explicit_rowvals(dxdotdx_explicit);
+            fdxdotdx_explicit(
+                dxdotdx_explicit.data(), t, N_VGetArrayPointer(x_pos),
+                state_.unscaledParameters.data(), state_.fixedParameters.data(),
+                state_.h.data(), w_.data());
+        }
+        fdxdotdw(t, x_pos);
+        /* Sparse matrix multiplication
+         dxdotdx_implicit += dxdotdw * dwdx */
+        dxdotdw_.sparse_multiply(dxdotdx_implicit, dwdx_);
+        
+        JSparse.sparse_add(dxdotdx_explicit, 1.0, dxdotdx_implicit, 1.0);
     } else {
         fJSparse(static_cast<SUNMatrixContent_Sparse>(SM_CONTENT_S(J)), t,
                  N_VGetArrayPointer(x_pos), state_.unscaledParameters.data(),
@@ -52,6 +64,7 @@ void Model_ODE::fJv(const realtype t, const AmiVector &x,
 void Model_ODE::fJv(N_Vector v, N_Vector Jv, realtype t, N_Vector x) {
     N_VConst(0.0, Jv);
     fJSparse(t, x, J_.get());
+    J_.refresh();
     J_.multiply(Jv, v);
 }
 
@@ -91,11 +104,12 @@ void Model_ODE::fJDiag(const realtype t, AmiVector &JDiag,
 }
 
 void Model_ODE::fdxdotdw(const realtype t, const N_Vector x) {
-    if (nw > 0 && ndxdotdw > 0) {
+    dxdotdw_.zero();
+    if (nw > 0 && dxdotdw_.capacity()) {
         auto x_pos = computeX_pos(x);
-        dxdotdw_.reset();
-        fdxdotdw_colptrs(dxdotdw_.indexptrs());
-        fdxdotdw_rowvals(dxdotdw_.indexvals());
+
+        fdxdotdw_colptrs(dxdotdw_);
+        fdxdotdw_rowvals(dxdotdw_);
         fdxdotdw(dxdotdw_.data(), t, N_VGetArrayPointer(x_pos),
                  state_.unscaledParameters.data(), state_.fixedParameters.data(),
                  state_.h.data(), w_.data());
@@ -105,27 +119,26 @@ void Model_ODE::fdxdotdw(const realtype t, const N_Vector x) {
 void Model_ODE::fdxdotdp(const realtype t, const N_Vector x) {
     auto x_pos = computeX_pos(x);
     fdwdp(t, N_VGetArrayPointer(x_pos));
-    fdxdotdw(t, x_pos);
 
     if (pythonGenerated) {
         // python generated
-        if (ndxdotdp_explicit > 0) {
-            dxdotdp_explicit.reset();
-            fdxdotdp_explicit_colptrs(dxdotdp_explicit.indexptrs());
-            fdxdotdp_explicit_rowvals(dxdotdp_explicit.indexvals());
+        dxdotdp_explicit.zero();
+        dxdotdp_implicit.zero();
+        if (dxdotdp_explicit.capacity()) {
+            fdxdotdp_explicit_colptrs(dxdotdp_explicit);
+            fdxdotdp_explicit_rowvals(dxdotdp_explicit);
             fdxdotdp_explicit(
                 dxdotdp_explicit.data(), t, N_VGetArrayPointer(x_pos),
                 state_.unscaledParameters.data(), state_.fixedParameters.data(),
                 state_.h.data(), w_.data());
         }
-        if (nw > 0 && ndxdotdp_implicit > 0) {
-            /* Sparse matrix multiplication
-             dxdotdp_implicit += dxdotdw * dwdp */
-            dxdotdp_implicit.reset();
-            fdxdotdp_implicit_colptrs(dxdotdp_implicit.indexptrs());
-            fdxdotdp_implicit_rowvals(dxdotdp_implicit.indexvals());
-            dxdotdw_.sparse_multiply(&dxdotdp_implicit, &dwdp_);
-        }
+        
+        fdxdotdw(t, x_pos);
+        /* Sparse matrix multiplication
+         dxdotdp_implicit += dxdotdw * dwdp */
+        dxdotdw_.sparse_multiply(dxdotdp_implicit, dwdp_);
+        
+        dxdotdp_full.sparse_add(dxdotdp_explicit, 1.0, dxdotdp_implicit, 1.0);
     } else {
         // matlab generated
         for (int ip = 0; ip < nplist(); ip++) {
@@ -147,16 +160,6 @@ std::unique_ptr<Solver> Model_ODE::getSolver() {
     return std::unique_ptr<Solver>(new amici::CVodeSolver());
 }
 
-void Model_ODE::fJB(realtype * /*JB*/, const realtype /*t*/,
-                    const realtype * /*x*/, const realtype * /*p*/,
-                    const realtype * /*k*/, const realtype * /*h*/,
-                    const realtype * /*xB*/, const realtype * /*w*/,
-                    const realtype * /*dwdx*/) {
-    throw AmiException("Requested functionality is not supported as %s is not "
-                       "implemented for this model!",
-                       __func__);
-}
-
 void Model_ODE::fJSparse(SUNMatrixContent_Sparse /*JSparse*/,
                          const realtype /*t*/, const realtype * /*x*/,
                          const realtype * /*p*/, const realtype * /*k*/,
@@ -176,59 +179,18 @@ void Model_ODE::fJSparse(realtype * /*JSparse*/, const realtype /*t*/,
                        __func__); // not implemented
 }
 
-void Model_ODE::fJSparse_colptrs(sunindextype * /*indexptrs*/) {
+void Model_ODE::fJSparse_colptrs(SUNMatrixWrapper &/*JSparse*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fJSparse_rowvals(sunindextype * /*indexvals*/) {
+void Model_ODE::fJSparse_rowvals(SUNMatrixWrapper &/*JSparse*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fJSparseB(SUNMatrixContent_Sparse /*JSparseB*/,
-                          const realtype /*t*/, const realtype * /*x*/,
-                          const realtype * /*p*/, const realtype * /*k*/,
-                          const realtype * /*h*/, const realtype * /*xB*/,
-                          const realtype * /*w*/, const realtype * /*dwdx*/) {
-    throw AmiException("Requested functionality is not supported as %s "
-                       "is not implemented for this model!",
-                       __func__); // not implemented
-}
-
-void Model_ODE::fJSparseB(realtype * /*JSparseB*/, const realtype /*t*/,
-                          const realtype * /*x*/, const realtype * /*p*/,
-                          const realtype * /*k*/, const realtype * /*h*/,
-                          const realtype * /*xB*/, const realtype * /*w*/,
-                          const realtype * /*dwdx*/) {
-    throw AmiException("Requested functionality is not supported as %s "
-                       "is not implemented for this model!",
-                       __func__); // not implemented
-}
-
-void Model_ODE::fJSparseB_colptrs(sunindextype * /*indexptrs*/) {
-    throw AmiException("Requested functionality is not supported as %s "
-                       "is not implemented for this model!",
-                       __func__); // not implemented
-}
-
-void Model_ODE::fJSparseB_rowvals(sunindextype * /*indexvals*/) {
-    throw AmiException("Requested functionality is not supported as %s "
-                       "is not implemented for this model!",
-                       __func__); // not implemented
-}
-
-void Model_ODE::fJDiag(realtype * /*JDiag*/, const realtype /*t*/,
-                       const realtype * /*x*/, const realtype * /*p*/,
-                       const realtype * /*k*/, const realtype * /*h*/,
-                       const realtype * /*w*/, const realtype * /*dwdx*/) {
-    throw AmiException("Requested functionality is not supported as %s is not "
-                       "implemented for this model!",
-                       __func__);
-}
-
 void Model_ODE::froot(realtype * /*root*/, const realtype /*t*/,
                       const realtype * /*x*/, const realtype * /*p*/,
                       const realtype * /*k*/, const realtype * /*h*/) {
@@ -256,25 +218,34 @@ void Model_ODE::fdxdotdp_explicit(realtype * /*dxdotdp_explicit*/, const realtyp
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdp_explicit_colptrs(sunindextype * /*indexptrs*/) {
+void Model_ODE::fdxdotdp_explicit_colptrs(SUNMatrixWrapper &/*dxdotdp*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdp_explicit_rowvals(sunindextype * /*indexvals*/) {
+void Model_ODE::fdxdotdp_explicit_rowvals(SUNMatrixWrapper &/*dxdotdp*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdp_implicit_colptrs(sunindextype * /*indexptrs*/) {
+void Model_ODE::fdxdotdx_explicit(realtype * /*dxdotdx_explicit*/, const realtype /*t*/,
+                                  const realtype * /*x*/, const realtype * /*p*/,
+                                  const realtype * /*k*/, const realtype * /*h*/,
+                                  const realtype * /*w*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdp_implicit_rowvals(sunindextype * /*indexvals*/) {
+void Model_ODE::fdxdotdx_explicit_colptrs(SUNMatrixWrapper &/*dxdotdx*/) {
+    throw AmiException("Requested functionality is not supported as %s "
+                       "is not implemented for this model!",
+                       __func__); // not implemented
+}
+
+void Model_ODE::fdxdotdx_explicit_rowvals(SUNMatrixWrapper &/*dxdotdx*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
@@ -289,13 +260,13 @@ void Model_ODE::fdxdotdw(realtype * /*dxdotdw*/, const realtype /*t*/,
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdw_colptrs(sunindextype * /*indexptrs*/) {
+void Model_ODE::fdxdotdw_colptrs(SUNMatrixWrapper &/*dxdotdw*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
 }
 
-void Model_ODE::fdxdotdw_rowvals(sunindextype * /*indexvals*/) {
+void Model_ODE::fdxdotdw_rowvals(SUNMatrixWrapper &/*dxdotdw*/) {
     throw AmiException("Requested functionality is not supported as %s "
                        "is not implemented for this model!",
                        __func__); // not implemented
@@ -308,14 +279,12 @@ void Model_ODE::fJB(const realtype t, realtype /*cj*/, const AmiVector &x,
     fJB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), JB);
 }
 
-void Model_ODE::fJB(realtype t, N_Vector x, N_Vector xB, N_Vector /*xBdot*/,
+void Model_ODE::fJB(realtype t, N_Vector x, N_Vector /*xB*/, N_Vector /*xBdot*/,
                     SUNMatrix JB) {
-    auto x_pos = computeX_pos(x);
-    fdwdx(t, N_VGetArrayPointer(x_pos));
-    SUNMatZero(JB);
-    fJB(SM_DATA_D(JB), t, N_VGetArrayPointer(x_pos),
-        state_.unscaledParameters.data(), state_.fixedParameters.data(),
-        state_.h.data(), N_VGetArrayPointer(xB), w_.data(), dwdx_.data());
+    fJSparse(t, x, J_.get());
+    J_.refresh();
+    auto JDenseB = SUNMatrixWrapper(JB);
+    J_.transpose(JDenseB, -1.0, nxtrue_solver);
 }
 
 void Model_ODE::fJSparseB(const realtype t, realtype /*cj*/, const AmiVector &x,
@@ -325,46 +294,33 @@ void Model_ODE::fJSparseB(const realtype t, realtype /*cj*/, const AmiVector &x,
     fJSparseB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), JB);
 }
 
-void Model_ODE::fJSparseB(realtype t, N_Vector x, N_Vector xB,
+void Model_ODE::fJSparseB(realtype t, N_Vector x, N_Vector /*xB*/,
                           N_Vector /*xBdot*/, SUNMatrix JB) {
-    auto x_pos = computeX_pos(x);
-    fdwdx(t, N_VGetArrayPointer(x_pos));
-    SUNMatZero(JB);
-    if (pythonGenerated) {
-        fJSparseB(SM_DATA_S(JB), t, N_VGetArrayPointer(x_pos),
-                  state_.unscaledParameters.data(), state_.fixedParameters.data(),
-                  state_.h.data(), N_VGetArrayPointer(xB), w_.data(),
-                  dwdx_.data());
-        fJSparseB_colptrs(SM_INDEXPTRS_S(JB));
-        fJSparseB_rowvals(SM_INDEXVALS_S(JB));
-    } else {
-        fJSparseB(static_cast<SUNMatrixContent_Sparse>(SM_CONTENT_S(JB)), t,
-                  N_VGetArrayPointer(x_pos), state_.unscaledParameters.data(),
-                  state_.fixedParameters.data(), state_.h.data(),
-                  N_VGetArrayPointer(xB), w_.data(), dwdx_.data());
-    }
+    fJSparse(t, x, J_.get());
+    J_.refresh();
+    auto JSparseB = SUNMatrixWrapper(JB);
+    J_.transpose(JSparseB, -1.0, nxtrue_solver);
 }
 
 void Model_ODE::fJDiag(realtype t, N_Vector JDiag, N_Vector x) {
-    auto x_pos = computeX_pos(x);
-    fdwdx(t, N_VGetArrayPointer(x_pos));
-    N_VConst(0.0, JDiag);
-    fJDiag(N_VGetArrayPointer(JDiag), t, N_VGetArrayPointer(x_pos),
-           state_.unscaledParameters.data(), state_.fixedParameters.data(),
-           state_.h.data(), w_.data(), dwdx_.data());
+    fJSparse(t, x, J_.get());
+    J_.refresh();
+    J_.to_diag(JDiag);
 }
 
 void Model_ODE::fJvB(N_Vector vB, N_Vector JvB, realtype t, N_Vector x,
                      N_Vector xB) {
     N_VConst(0.0, JvB);
-    fJSparseB(t, x, xB, nullptr, J_.get());
-    J_.multiply(JvB, vB);
+    fJSparseB(t, x, xB, nullptr, JB_.get());
+    JB_.refresh();
+    JB_.multiply(JvB, vB);
 }
 
 void Model_ODE::fxBdot(realtype t, N_Vector x, N_Vector xB, N_Vector xBdot) {
     N_VConst(0.0, xBdot);
-    fJSparseB(t, x, xB, nullptr, J_.get());
-    J_.multiply(xBdot, xB);
+    fJSparseB(t, x, xB, nullptr, JB_.get());
+    JB_.refresh();
+    JB_.multiply(xBdot, xB);
 }
 
 void Model_ODE::fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot) {
@@ -374,10 +330,7 @@ void Model_ODE::fqBdot(realtype t, N_Vector x, N_Vector xB, N_Vector qBdot) {
 
     if (pythonGenerated) {
         /* call multiplication */
-        if (ndxdotdp_explicit > 0)
-            dxdotdp_explicit.multiply(qBdot, xB, state_.plist, true);
-        if (ndxdotdp_implicit > 0)
-            dxdotdp_implicit.multiply(qBdot, xB, state_.plist, true);
+        dxdotdp_full.multiply(qBdot, xB, state_.plist, true);
         N_VScale(-1.0, qBdot, qBdot);
     } else {
         /* was matlab generated */
@@ -403,7 +356,7 @@ void Model_ODE::fxBdot_ss(realtype /*t*/, N_Vector xB, N_Vector xBdot) const {
     /* Right hande side of the adjoint state for steady state computations.
        J is fixed (as x remeins in steady state), so the RHS becomes simple. */
     N_VConst(0.0, xBdot);
-    J_.multiply(xBdot, xB);
+    JB_.multiply(xBdot, xB);
 }
 
 void Model_ODE::fqBdot_ss(realtype /*t*/, N_Vector xB, N_Vector qBdot) const {
@@ -414,7 +367,8 @@ void Model_ODE::fqBdot_ss(realtype /*t*/, N_Vector xB, N_Vector qBdot) const {
 
 void Model_ODE::fJSparseB_ss(SUNMatrix JB) {
     /* Just copy the model Jacobian */
-    SUNMatCopy(J_.get(), JB);
+    SUNMatCopy(JB_.get(), JB);
+    JB_.refresh();
 }
 
 void Model_ODE::writeSteadystateJB(const realtype t, realtype /*cj*/,
@@ -422,9 +376,11 @@ void Model_ODE::writeSteadystateJB(const realtype t, realtype /*cj*/,
                                    const AmiVector &xB, const AmiVector & /*dxB*/,
                                    const AmiVector &xBdot) {
     /* Get backward Jacobian */
-    fJSparseB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(), J_.get());
+    fJSparseB(t, x.getNVector(), xB.getNVector(), xBdot.getNVector(),
+              JB_.get());
+    JB_.refresh();
     /* Switch sign, as we integrate forward in time, not backward */
-    J_.scale(-1);
+    JB_.scale(-1);
 }
 
 void Model_ODE::fsxdot(const realtype t, const AmiVector &x,
@@ -444,6 +400,7 @@ void Model_ODE::fsxdot(realtype t, N_Vector x, int ip, N_Vector sx,
         // the same for all remaining
         fdxdotdp(t, x);
         fJSparse(t, x, J_.get());
+        J_.refresh();
     }
     if (pythonGenerated) {
         /* copy dxdotdp and the implicit version over */
@@ -451,23 +408,9 @@ void Model_ODE::fsxdot(realtype t, N_Vector x, int ip, N_Vector sx,
         N_VConst(0.0, sxdot);
         realtype *sxdot_tmp = N_VGetArrayPointer(sxdot);
 
-        // copy explicit version
-        if (ndxdotdp_explicit > 0) {
-            auto col_exp = dxdotdp_explicit.indexptrs();
-            auto row_exp = dxdotdp_explicit.indexvals();
-            auto data_exp_ptr = dxdotdp_explicit.data();
-            for (sunindextype i = col_exp[plist(ip)]; i < col_exp[plist(ip) + 1]; ++i)
-                sxdot_tmp[row_exp[i]] += data_exp_ptr[i];
-        }
-
-        // copy implicit version
-        if (ndxdotdp_implicit > 0) {
-            auto col_imp = dxdotdp_implicit.indexptrs();
-            auto row_imp = dxdotdp_implicit.indexvals();
-            auto data_imp_ptr = dxdotdp_implicit.data();
-            for (sunindextype i = col_imp[plist(ip)]; i < col_imp[plist(ip) + 1]; ++i)
-                sxdot_tmp[row_imp[i]] += data_imp_ptr[i];
-        }
+        dxdotdp_full.scatter(plist(ip), 1.0, nullptr,
+                             gsl::make_span(sxdot_tmp, nx_solver),
+                             0, nullptr, 0);
 
     } else {
         /* copy dxdotdp over */
